# -----------------------------------------------------------------------------
# Chimera script to make image of single icos virus capsid from PDB with
# BIOMT records
#
# Covering or iconifying the Chimera window while it is rendering images
# will corrupt the image.  
#
# based on code written for Chimera version 1.2199 by Tom Goddard, 3/27/06,
# (goddard@cgl.ucsf.edu). -- C. Lawson August 2006
# 
# updated 15 June 2011 with changes from Batsal Devkota

from os      import environ
from os.path import join, basename
from chimera import viewer, runCommand, openModels
import chimera

pdb_file = environ["CHIMERAPDB"]

red = (1,0,0,1)         # Red, green, blue, opacity
green = (0,1,0,1)
blue = (0,0,1,1)
cyan = (0,1,1,1)
yellow=(1,1,0,1)
magenta=(1,0,1,1)
grey=(1,1,1,1)
resolution = 5          # Surface resolution, angstroms.
dens = 0.02             # Controls surface size.  Atoms / A^3.
dens_ca_only = 0.002    # Controls surface size for CA chains.
supersample = 3         # Capture images N times bigger for better quality.
padding = .05           # Fraction of window size for padding at edge.

import MultiScale
d = MultiScale.multiscale_model_dialog(create = True)
d.surface_resolution.set('%.4g' % resolution)
d.density_threshold.set('%.6g' % dens)
d.density_threshold_ca_only.set('%.6g' % dens_ca_only)
d.multimer_type.set(d.multimer_biounit)
viewer.windowSize = (512, 512)  # Set Chimera window size
runCommand('set bg_color white')


def make_images(pdb_file):
     # open structure
     molecule = openModels.open(pdb_file, 'PDB')[0]

     #delete solvent
     runCommand('delete solvent')

     # Create multiscale surfaces
     d.make_multimers([molecule])

     #--------begin new commands added 2011----------------- 
     # Show only surfaces, even for original copy of molecule.
     d.select_all_chains_cb()
     d.show_only_surface_cb()
     d.clear_selection_cb()

     #Turn on silhouette edges 
     runCommand('set silhouette')

     #Use high quality lighting, requires Chimera v 1.4 or higher
     runCommand('set light_quality glossy')
     #--------end new commands added 2011----------------- 

     # turn model (better view for helical viruses)
     runCommand('turn x -90')

     # Set camera so virus fills window.
     set_camera_parameters(padding)

     #color 10 subunits by matrix index
     # 1 = yellow  2-5 = green, 6-10 = red
     # 11-20 = dark blue  21-40 = cyan  41-60 = white/grey
     #color_multiscale(yellow, 1, range(1,2))  
     #color_multiscale(green, 1, range(2,6))   
     #color_multiscale(red, 1, range(6,11)) 
     # color_multiscale(blue, 1, range(11,21))
     # color_multiscale(cyan, 1, range(21,41))
     # color_multiscale(grey, 1, range(41,61)) 

     # Capture and save image
     image_path = join(pdb_file + '.jpg')
     from chimera.printer import saveImage
     saveImage(image_path, format='jpeg', supersample = supersample)


# ---------------------------------------------------------------------------
# This adjusts camera so virus capsid fills window.
#
def set_camera_parameters(padding):

    have_bbox, bbox = chimera.openModels.bbox()
    if not have_bbox:
        return False

    z1 = bbox.llf.z
    z2 = bbox.urb.z             # z2 > z1
    zsize = z2 - z1
    xsize = bbox.urb.x - bbox.llf.x
    ysize = bbox.urb.y - bbox.llf.y

    v = chimera.viewer
    c = v.camera
# new
    s = .5 * max(xsize, ysize)
    s += padding * s
    v.viewSize = s
#end new
    #v.viewSize = .5 * max(xsize, ysize)
    v.scaleFactor = 1
    c.center = bbox.center().data()
    c.viewDistance = zsize
    c.nearFar = (z2 + padding*zsize, z1 - padding*zsize)

    return True
# -----------------------------------------------------------------------------
# Color certain matrix subunits and chains of a multiscale model.
#

def color_multiscale(rgba, model_num, matrix_nums, chain_ids = None):


  import MultiScale
  d = MultiScale.multiscale_model_dialog(True)
  for cp in d.chain_pieces():
    g = cp.surface_group
    if g.model().id == model_num:
      mnum, chain_id = g.oslName.split('.')
      if chain_ids == None or chain_id in chain_ids:
        if int(mnum) in matrix_nums:
          cp.set_color(rgba)


# ----------------------------------------------------------------------------
#
# This function call makes the image
#
make_images(pdb_file)

#Exit Chimera
runCommand('stop noask')

